/* 
 * Copyright (C) 2010, Motorola India Private Ltd., 
 * All Rights Reserved 
 * Class name: WSConnector.java 
 * Description: This is the ESB service which exposes the WSDL interface for the WS aware clients
 * for sending provisioning requests, sagrequest.xsd, sagresponse.xsd and fault.xsd are used for the
 * WSDL contract.
 * It also exposes HTTP endpoint which is used by HTTP clients to send provisioning requests to SAG.
 * Modification History: 
 **********************************************************
 * Date          Author		    Comments
 * Sept 08,2010 Shravan		Created
 * Jan 15, 2010  Shravan		enhanced the logging
 **********************************************************
 */
package com.mot.sag.nbi;


import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;
import org.jboss.soa.esb.actions.AbstractActionPipelineProcessor;
import org.jboss.soa.esb.actions.ActionProcessingException;
import org.jboss.soa.esb.client.ServiceInvoker;
import org.jboss.soa.esb.helpers.ConfigTree;
import org.jboss.soa.esb.listeners.message.MessageDeliverException;
import org.jboss.soa.esb.message.Message;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.mot.sag.exception.MPSException;
import com.mot.sag.logger.MPSLogger;
import com.mot.sag.nbi.util.SAGNBIConstants;
import com.mot.sag.util.XMLUtil;

public class WSConnector extends AbstractActionPipelineProcessor
{
	protected ConfigTree _config;
	private static Logger logger = MPSLogger.getLogger(WSConnector.class);
	final static String SPREQUESTMESSAGE = "mps:SPRequestMessage";
	final static String SPREQUESTPART = "SPRequestPart";
	final static String XMLNS = "xmlns";
	final static String INBOUND = "inbound.xsd";
	String responseMsg ="<x1:SPResponseMessage xmlns:x1=\"http://motorola.com/oss/sag\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://motorola.com/oss/sag types/sagrequest.xsd \"><x1:arg0>"
		+"SPRequestMessage is not available in the soap request"+"</x1:arg0></x1:SPResponseMessage>";

	public WSConnector(ConfigTree config)
	{
		_config = config;
	}
	public void processException(Message message, java.lang.Throwable e)
	{
		logger.error("Exception in SAGWebService action", e);
	}

	public void processSuccess(Message message)
	{
		logger.debug("SOAP request successfully dispatched to Validation service");
	}
	public Message process(Message message) throws ActionProcessingException
	{
		//logger.debug("org.jboss.soa.esb.gateway.original.url="+ message.getProperties().getProperty("org.jboss.soa.esb.gateway.original.url"));
		//logger.debug("org.jboss.soa.esb.message.source="+ message.getProperties().getProperty("org.jboss.soa.esb.message.source"));
		logger.debug("org.jboss.soa.esb.message.time.dob="+ message.getProperties().getProperty("org.jboss.soa.esb.message.time.dob"));
		//logger.debug("org.jboss.soa.esb.gateway.original.file.name="+ message.getProperties().getProperty("org.jboss.soa.esb.gateway.original.file.name"));
		//logger.debug("org.jboss.soa.esb.gateway.file="+ message.getProperties().getProperty("org.jboss.soa.esb.gateway.file"));
		
		String request = (String)message.getBody().get() ;	
		String xml = null;
		if (logger.isDebugEnabled())
		{
			logger.debug("Received SOAP request : \n" + request) ;
			
		}
		//message.getBody().add("soap_original_request", request);
		request = StringEscapeUtils.unescapeXml(request);	
		if (logger.isDebugEnabled())
		{
			logger.debug("unescaped request \n" + request) ; 
			
		}
		message.getBody().add(request);
		
			String result = message.getBody().get().toString();
			//logger.debug("---------------result=\n" + result);
			//String spRequestMessagexml = null;
			int beginIndex = result.indexOf("<"+SPREQUESTMESSAGE);
			int endIndex = result.indexOf(SPREQUESTMESSAGE+">");
			try {
				Document sprmfromSOAPbody = XMLUtil.getDocFromXmlStr(result);
				NodeList nodes = sprmfromSOAPbody.getElementsByTagNameNS("*",SPREQUESTMESSAGE);
				if(nodes != null && nodes.getLength() > 0)
                {
					  logger.info(" found element with " + SPREQUESTMESSAGE);               
              	  // this logic is invoked when SPR is posted using Web Service here we remove the method specific tag and extract the SPR and palce it in 0 part in message 
					  // and name the part to "SPRequestPart" so that SAG does the validation and processing.
					  
					  Node  node = nodes.item(0);
              	  xml = node.getFirstChild().getNodeValue();
              	  
              	  if(xml == null){
              		    node = nodes.item(1);
                  	  xml = node.getFirstChild().getNodeValue();
                  	  logger.info(" xml is null from the part 0 trying for part1");
              	  }
              	 
              	  logger.info(" xml being added to message part is \n" + xml);
              	 
                }else {
              	  logger.debug("not found any element with " + SPREQUESTMESSAGE);     
              	   // this logic is invoked when SPR is posted using HTTP client Service here we create part with content ID "SPRequestPart" so that SAG does the validation and processing.
                   
                }
                
			} catch (MPSException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			if(beginIndex == -1 || endIndex == -1){
				//prepare soap fault saying sprequestmessage is not available
				logger.debug(" not found element with name " + SPREQUESTMESSAGE +"from the received soap request");
				message.getBody().add(responseMsg);
				return message;
			}			
			//endIndex = endIndex+SPREQUESTMESSAGE.length()+1;
			//spRequestMessagexml = result.substring(beginIndex, endIndex);
			//spRequestMessagexml = spRequestMessagexml.replaceAll(XMLNS, " "+XMLNS);
			//spRequestMessagexml = spRequestMessagexml.replaceAll(INBOUND, " "+INBOUND);
			//spRequestMessagexml = spRequestMessagexml.trim();
			//logger.debug("required string \n" + spRequestMessagexml);

	
		try{

			message.getProperties().setProperty(SAGNBIConstants.NBI_MODE, SAGNBIConstants.NBI_MODE_WS);
			//Util.getRequestHandler().getEsbServiceCategory()
			new ServiceInvoker(SAGNBIConstants.CORE_SERVICECAT_VALIDATION, SAGNBIConstants.CORE_SERVICE_VALIDATION).deliverAsync(message);


		}  catch (MessageDeliverException e) {
			// Failed to send the response handling service.
			throw new ActionProcessingException(e);
		} 
		return null;

	}	
}
